

/*
 * 
 *  file:  ./src/edu/virginia/bioch/nopt/alignments/Alignment.java
 * 
 *  Copyright (c) 2004,  the University of Virginia.
 *  All rights reverved.
 * 
 *  See the file COPYRIGHT in the top directory of this distribution for
 *  more information.
 *  
 *  THE SOFTWARE IS PROVIDED _AS IS_, WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 *  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 *  DEALINGS IN THE SOFTWARE.  
 *  
 */ 

package edu.virginia.bioch.nopt.alignments;



import java.util.*;
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;                                                   
import javax.swing.*;
import javax.swing.event.*;

import edu.virginia.bioch.util.*;
import edu.virginia.bioch.nopt.alignments.*;
import edu.virginia.bioch.scoringmatrices.*;
import edu.virginia.bioch.nopt.options.*;
import edu.virginia.bioch.nopt.conditions.*;
import edu.virginia.bioch.nopt.alignments.*;


public class AlignmentParameters
{
	protected Map valMap; 
	protected String key;
	protected ScoringMatrix sm;

	//
	// Constructor
	//
	public AlignmentParameters(String s)
		throws Exception
	{
		System.out.println("AlignmentParameters constructor");
		
		valMap = new HashMap();

		if ( s != null )
		{
			StringTokenizer t = new StringTokenizer(s,"|");
			key = t.nextToken();
		
			while ( t.hasMoreTokens() )
			{
				StringTokenizer tt = new StringTokenizer( t.nextToken(), "=");
				String name = tt.nextToken();
				String value = tt.nextToken();
				valMap.put(name,value);
			}
		}
		else
		{
			throw new Exception("No param string for AlignmentParameters!");
		}

		sm = new ScoringMatrix( (String)(valMap.get("scoringMatrix")) );
	} 

	public String getKey()
	{ return key; }

	public int getGapCreate() 
	{ return Integer.parseInt((String)(valMap.get("gapCreate"))); }

	public int getGapExtend() 
	{ return Integer.parseInt((String)(valMap.get("gapExtend"))); }

	public String getScoringMatrixName() 
	{ return (String)(valMap.get("scoringMatrix")); }

	public ScoringMatrix getScoringMatrix() 
	{ return sm; }

	public int getOptimalScore() 
	{ return Integer.parseInt((String)(valMap.get("optimalScore"))); }

	public float getLowerBoundPercent() 
	{ return Float.parseFloat((String)(valMap.get("lowerBoundPercent"))); }

	public float getUpperBoundPercent() 
	{ return Float.parseFloat((String)(valMap.get("upperBoundPercent"))); }

	public int getLowerBound() 
	{ return Integer.parseInt((String)(valMap.get("lowerBound"))); }

	public int getUpperBound() 
	{ return Integer.parseInt((String)(valMap.get("upperBound"))); }

	public int getMaxRobust() 
	{ return Integer.parseInt((String)(valMap.get("maxRobust"))); }

	public int getMinRobust() 
	{ return Integer.parseInt((String)(valMap.get("minRobust"))); }

	public String getSeq1Name()
	{ 
		String n = (String)(valMap.get("seq1Name")); 
		if ( n == null )
			return "sequence1";
		else
			return n;
	}

	public String getSeq2Name()
	{ 
		String n = (String)(valMap.get("seq2Name")); 
		if ( n == null )
			return "sequence2";
		else
			return n;
	}

	public String getSeq1Description()
	{ 
		String n = (String)(valMap.get("seq1Description")); 
		if ( n == null )
			return "sequence1 desc";
		else
			return n;
	}

	public String getSeq2Description()
	{ 
		String n = (String)(valMap.get("seq2Description")); 
		if ( n == null )
			return "sequence2 desc";
		else
			return n;
	}

	public String getSeq1()
	{ return (String)(valMap.get("seq1")); }

	public String getSeq2()
	{ return (String)(valMap.get("seq2")); }


	public boolean isDNA()
	{
		int x = Integer.parseInt((String)(valMap.get("dna")));
		if ( x == 0 )
			return false;
		else
			return true;
	}

	public boolean isLocal()
	{
		int x = Integer.parseInt((String)(valMap.get("local")));
		if ( x == 0 )
			return false;
		else
			return true;
	}

	public String getAlgorithmName()
	{ return (String)(valMap.get("algorithm")); }

}
